;Whlscheibentelefon meets Handyelektronik
;2007 Bjrn Wieck
;
;Whlton 440Hz wird durch einen Astabilen Multivibrator auf den Hrerlautsprecher geschaltet

.include "m8def.inc"

;Registerdefinitionen
.def temp1=		r16
.def temp2=		r17
.def temp3=		r18
.def temp4=		r19
.def debug=		r20
;.equ quartz = 3686400 ; fr Standard STK500
.equ quartz=	2000000			;Interner Oszillator 2MHz
.equ baud=		4800

;IRQ-Sprungtabelle
.org 0x00
		rjmp RESET				; Reset Handler
		rjmp EXT_INT0			; IRQ0 Handler
		rjmp EXT_INT1			; IRQ1 Handler
		reti ;rjmp TIM2_COMP	; Timer2 Compare Handler
		reti ;rjmp TIM2_OVF		; Timer2 Overflow Handler
		reti ;rjmp TIM1_CAPT	; Timer1 Capture Handler
		reti ;rjmp TIM1_COMPA	; Timer1 CompareA Handler
		reti ;rjmp TIM1_COMPB	; Timer1 CompareB Handler
		reti ;rjmp TIM1_OVF		; Timer1 Overflow Handler
		reti ;rjmp TIM0_OVF		; Timer0 Overflow Handler
		reti ;rjmp SPI_STC		; SPI Transfer Complete Handler
		reti ;rjmp USART_RXC	; USART RX Complete Handler
		reti ;rjmp USART_UDRE	; UDR Empty Handler
		reti ;rjmp USART_TXC	; USART TX Complete Handler
		reti ;rjmp ADC			; ADC Conversion Complete Handler
		reti ;rjmp EE_RDY		; EEPROM Ready Handler
		reti ;rjmp ANA_COMP		; Analog Comparator Handler
		reti ;rjmp TWSI			; Two-wire Serial Interface Handler
		reti ;rjmp SPM_RDY		; Store Program Memory Ready Handler

;Portkonfiguration
;PD0 > Handypowerschalter			OUT 
;PD1 > Debug						(TX) Ausgabe fr debugging
;PD2 > Gabelschalter INT0			IN High-Aktiv
;PD3 > Klingelsignal INT1			IN High-aktiv
;PD4 > Whlscheibe Pulse			IN Low-Aktiv
;PD5 > Whlscheibe Enable			IN High-Aktiv
;PD6 > Taste 3	7					OUT
;PD7 > Wahlende						IN High-Aktiv

;PB0 > Taste 0				OUT
;PB1 > Taste 1	9			OUT
;PB2 > Taste 2	8			OUT
;PB3 > 440Hz Ton			OUT
;PB4 > Taste 4	6			OUT
;PB5 > Taste 5	5			OUT

;PC0 > Taste 6	4			OUT
;PC1 > Taste 7	3			OUT
;PC2 > Taste 8	2			OUT
;PC3 > Taste 9	1			OUT
;PC4 > Taste Grn			OUT
;PC5 > Taste Rot			OUT


RESET:
;Ports einstellen
					ldi temp1, 0b01000001
					out DDRD, temp1
					ldi temp1, 0b00111111
					out DDRB, temp1
					out DDRC, temp1
					ldi temp1, 0
					out PORTB, temp1
					out PORTC, temp1

;Stack initalisieren
					ldi temp1, high(RAMEND)
					out SPH, temp1
					ldi temp1, low(RAMEND)
					out SPL, temp1
;IRQs einschalten Sleep-Mode Idle
					ldi temp1, 0b0001111
					out MCUCR, temp1
					ldi temp1, 0b11000000
					out GICR, temp1

;Counter 0 fr Nummernwhler konfigurieren
					ldi temp1, 0b00000110
					out TCCR0, temp1
					ldi temp1, 0
					out TCNT0, temp1

;Counter 1 fr Klingelwarteschleife konfigurieren
					ldi temp1, 0b00000000
					out TCCR1A, temp1
					ldi temp1, 0b00000101
					out TCCR1B, temp1

;UART Initalisierung
					sbi UCSRB, TXEN		;enable transmit
					ldi temp1, quartz / (baud * 16) - 1
					out UBRRL, temp1	;BAUD Rate

;PWM fr Counter_2 initalisieren, Vorteiler 8 440Hz@1MHz
					ldi temp1, 1<<COM21 | 1<<WGM20 | 0<<CS22 | 1<<CS21 | 0<<CS20
					out TCCR2, temp1
					ldi temp1, 0		;440 Hz Ton ausschalten
					out OCR2, temp1

;Handy einschalten und auf Netz warten 
					sbi PORTD, 0
rcall uart_power
					rcall wait100ms
					rcall wait100ms
					rcall wait100ms
					rcall wait100ms
					rcall wait100ms
					rcall wait100ms
					rcall wait100ms
					rcall wait100ms
					rcall wait100ms
					rcall wait100ms
					cbi PORTD, 0
					ldi temp1, 0
					out TCNT1H, temp1	;Warteschleife zurcksetzen
					out TCNT1L, temp1
Handypwr:			in temp1, TCNT1L
					in temp1, TCNT1H
					cpi temp1, 140		;Warten auf D-Netz
					brne Handypwr

;Interrups einschalten
					sei

;Hauptschleife
rcall uart_sleep
loop:				sleep
					rjmp loop



;Anrufen
EXT_INT0:								;Hrer wurde abgenommen
					cli
rcall uart_G1
					ldi temp3, 128
					out OCR2, temp3		;440Hz Ton einschalten
rcall uart_440on
NwEn:				sbis PIND, 2		;ist Hrer wieder aufgelegt  ?
					rjmp Anrufende
					sbic PIND, 7		;letzte Zahl gewhlt
					rjmp Anrufen
					sbis PIND, 5		;warten auf Wahlscheibe Enable
					rjmp NwEn
					ldi temp1, 0		;Counter auf 0 setzen
					out OCR2, temp1		;440Hz Ton ausschalten
rcall uart_440off
					out TCNT0, temp1
NwDi:				sbic PIND, 5		;warten auf Wahlscheibe disable
					rjmp NwDi
					in temp1, TCNT0
Nr10:				cpi temp1, 10		;10 Pulse ?
					brne Nr9
					sbi PORTB,0			;Taste 0 aktivieren
					rcall wait100ms
ldi temp1, 0
rcall uart_number
					cbi PORTB, 0 
Nr9:				cpi temp1, 9
					brne Nr8
					sbi PORTB, 1
					rcall wait100ms
rcall uart_number
					cbi PORTB, 1
Nr8:				cpi temp1, 8
					brne Nr7
					sbi PORTB, 2
					rcall wait100ms
rcall uart_number
					cbi PORTB, 2
Nr7:				cpi temp1, 7
					brne Nr6
					sbi PORTD, 6
					rcall wait100ms
rcall uart_number
					cbi PORTD, 6
Nr6:				cpi temp1, 6
					brne Nr5
					sbi PORTB, 4
					rcall wait100ms
rcall uart_number
					cbi PORTB, 4
Nr5:				cpi temp1, 5
					brne Nr4
					sbi PORTB, 5
					rcall wait100ms
rcall uart_number
					cbi PORTB, 5
Nr4:				cpi temp1, 4
					brne Nr3
					sbi PORTC, 0
					rcall wait100ms
rcall uart_number
					cbi PORTC, 0
Nr3:				cpi temp1, 3
					brne Nr2
					sbi PORTC, 1
					rcall wait100ms
rcall uart_number
					cbi PORTC, 1
Nr2:				cpi temp1, 2
					brne Nr1
					sbi PORTC, 2
					rcall wait100ms
rcall uart_number
					cbi PORTC, 2
Nr1:				cpi temp1, 1
					brne NrX
					sbi PORTC, 3
					rcall wait100ms
rcall uart_number
					cbi PORTC, 3
NrX:				rjmp NwEn
Anrufen:			ldi temp3,0
					out OCR2, temp3			;440Hz Ton ausschalten
rcall uart_440off
					sbi PORTC, 5			;Grne Taste aktivieren
					rcall wait100ms
					cbi PORTC, 5
rcall uart_green
					rcall wait100ms
					rcall wait100ms
					rcall wait100ms
					rcall wait100ms
					rcall wait100ms
					rcall wait100ms
					rcall wait100ms
					rcall wait100ms
					rcall wait100ms
					rcall wait100ms
					rcall wait100ms
					rcall wait100ms
					rcall wait100ms
					rcall wait100ms
					rcall wait100ms
					rjmp NwEn				;zurck zur Abfrage
Anrufende:			ldi temp3, 0
					out OCR2, temp3			;440Hz Ton ausschalten
rcall uart_440off
rcall uart_G0					
					sbi PORTC, 4			;Rote Taste aktivieren
					rcall wait100ms
rcall uart_red
					cbi PORTC, 4
					in temp1, GIFR
					cbr temp1, 6
					cbr temp1, 7
					out GIFR, temp1			;IRQ-Flags lschen
rcall uart_sleep
					sei						;zurck zur Hauptschleife
					reti

;Klingelsignal
EXT_INT1:
					cli
rcall uart_ring
CNT1_RESET:			ldi temp1, 0
					out TCNT1H, temp1	;Warteschleife zurcksetzen
					out TCNT1L, temp1
					sbic PIND, 2		;Hrer abgenommen ?
					rjmp Sprechen
					sbic PIND, 3
					rjmp CNT1_RESET
Time_over:			
					sbic PIND, 2		;Hrer abgenommen ?
					rjmp Sprechen
					sbic PIND, 3
					rjmp CNT1_RESET
					in temp1, TCNT1L
					in temp1, TCNT1H
					cpi temp1, 76		;nach dem Letzten Klingelsignal 10 Sek warten (38)
					brne Time_over
					rjmp Sprechenende
Sprechen:			
rcall uart_G1
					sbi PORTC, 5		;Grne Taste aktivieren
					rcall wait100ms
rcall uart_green
					cbi PORTC, 5
Sprechen1:			sbic PIND, 2		;Hrer aufgelegt ?
					rjmp Sprechen1
rcall uart_G0
					sbi PORTC, 4		;Rote Taste aktivieren
					rcall wait100ms
rcall uart_red
					cbi PORTC, 4
Sprechenende:		in temp1, GIFR
					cbr temp1, 6
					cbr temp1, 7
					out GIFR, temp1		;IRQ-Flags lschen
rcall uart_sleep
					sei					;zurck zur Hauptschleife
					reti

;Warteschleife 100 Millisekunden
wait100ms:			ldi temp4, 2
					ldi temp3, 5
					ldi temp2, 0
wait100_1:			dec temp2
					brne wait100_1
					dec temp3
					brne wait100_1
					dec temp4
					brne wait100_1
					ret

;UART Debug

uart_G1:			ldi temp3, 12
					ldi ZL, LOW(text1*2)	;Z-Pointer auf text1 setzen Lowbyte
					ldi ZH, HIGH(text1*2)	;Z-Pointer auf text1 setzen Highbyte
getchar1:			lpm debug, Z+
					rcall sendbyte
					dec temp3
					brne getchar1
					rcall sendcrlf
					ret

uart_G0:			ldi temp3, 12
					ldi ZL, LOW(text2*2)	;Z-Pointer auf text1 setzen Lowbyte
					ldi ZH, HIGH(text2*2)	;Z-Pointer auf text1 setzen Highbyte
getchar2:			lpm debug, Z+
					rcall sendbyte
					dec temp3
					brne getchar2
					rcall sendcrlf
					ret

uart_red:			ldi temp3, 12
					ldi ZL, LOW(text5*2)	;Z-Pointer auf text1 setzen Lowbyte
					ldi ZH, HIGH(text5*2)	;Z-Pointer auf text1 setzen Highbyte
getchar3:			lpm debug, Z+
					rcall sendbyte
					dec temp3
					brne getchar3
					rcall sendcrlf
					ret

uart_green:			ldi temp3, 12
					ldi ZL, LOW(text4*2)	;Z-Pointer auf text1 setzen Lowbyte
					ldi ZH, HIGH(text4*2)	;Z-Pointer auf text1 setzen Highbyte
getchar4:			lpm debug, Z+
					rcall sendbyte
					dec temp3
					brne getchar4
					rcall sendcrlf
					ret

uart_number:		ldi temp3, 12
					ldi ZL, LOW(text3*2)	;Z-Pointer auf text1 setzen Lowbyte
					ldi ZH, HIGH(text3*2)	;Z-Pointer auf text1 setzen Highbyte
getchar5:			lpm debug, Z+
					rcall sendbyte
					dec temp3
					brne getchar5
					mov debug, temp1
					subi debug, 0xd0
					rcall sendbyte
					rcall sendcrlf
					ret

uart_ring:			ldi temp3, 12
					ldi ZL, LOW(text6*2)	;Z-Pointer auf text1 setzen Lowbyte
					ldi ZH, HIGH(text6*2)	;Z-Pointer auf text1 setzen Highbyte
getchar6:			lpm debug, Z+
					rcall sendbyte
					dec temp3
					brne getchar6
					rcall sendcrlf
					ret

uart_sleep:			ldi temp3, 12
					ldi ZL, LOW(text7*2)	;Z-Pointer auf text1 setzen Lowbyte
					ldi ZH, HIGH(text7*2)	;Z-Pointer auf text1 setzen Highbyte
getchar7:			lpm debug, Z+
					rcall sendbyte
					dec temp3
					brne getchar7
					rcall sendcrlf
					ret

uart_440on:			ldi temp3, 12
					ldi ZL, LOW(text8*2)	;Z-Pointer auf text1 setzen Lowbyte
					ldi ZH, HIGH(text8*2)	;Z-Pointer auf text1 setzen Highbyte
getchar8:			lpm debug, Z+
					rcall sendbyte
					dec temp3
					brne getchar8
					rcall sendcrlf
					ret

uart_440off:		ldi temp3, 12
					ldi ZL, LOW(text9*2)	;Z-Pointer auf text1 setzen Lowbyte
					ldi ZH, HIGH(text9*2)	;Z-Pointer auf text1 setzen Highbyte
getchar9:			lpm debug, Z+
					rcall sendbyte
					dec temp3
					brne getchar9
					rcall sendcrlf
					ret

uart_power:			ldi temp3, 12
					ldi ZL, LOW(text10*2)	;Z-Pointer auf text1 setzen Lowbyte
					ldi ZH, HIGH(text10*2)	;Z-Pointer auf text1 setzen Highbyte
getchar10:			lpm debug, Z+
					rcall sendbyte
					dec temp3
					brne getchar10
					rcall sendcrlf
					ret
sendcrlf:
					ldi debug, 13
					rcall sendbyte
					ldi debug, 10
					rcall sendbyte
					ret


sendbyte:
tx1:				sbis UCSRA,UDRE		;Warten, bis UDR bereit ist
					rjmp tx1
					out UDR, debug
					ret


.cseg
text1:				.db "Gabel oben ",0
text2:				.db "Gabel unten",0
text3:				.db "Nummer     ",0
text4:				.db "Gruen      ",0
text5:				.db "Rot        ",0
text6:				.db "Klingeln   ",0
text7:				.db "sleep...   ",0
text8:				.db "440Hz Ein  ",0
text9:				.db "440Hz Aus  ",0
text10:				.db "Handy Ein  ",0
